<?php

namespace Ulmus\Ldap;

use Ulmus\Adapter\AdapterInterface;

use Ulmus\Common\PdoObject;

class ConnectionAdapter extends \Ulmus\ConnectionAdapter
{
    public string $name;

    public array $configuration;

    protected AdapterInterface $adapter;

    private $connection;

    public function resolveConfiguration() : void
    {
        $connection = $this->configuration['connections'][$this->name] ?? [];

        if ( false !== ( $adapterName = $connection['adapter'] ?? false ) ) {
            $this->adapter = $this->instanciateAdapter($adapterName);
        }
        else {
            $this->adapter = new Adapter\Ldap();
        }
        
        $this->adapter->setup($connection);
    }

    public function getConfiguration() : array
    {
        return $this->configuration['connections'][$this->name];
    }

    /**
     * Connect the adapter
     * @return self
     */
    public function connect() : self
    {
        $this->connection = $this->adapter->connect();
        
        return $this;
    }

    public function connector() : object
    {
        return $this->connection;
    }
    
    public function adapter() : AdapterInterface
    {
        return $this->adapter;
    }

    /**
     * Instanciate an adapter which interact with the data source
     * @param  string $name An Ulmus adapter or full class name implementing AdapterInterface
     * @return AdapterInterface
     */
    protected function instanciateAdapter($name) : AdapterInterface
    {
        $class = substr($name, 0, 2) === "\\" ? $name : "\\Ulmus\\Adapter\\$name";
        
        return new $class();
    }
}
