<?php

namespace Ulmus\Ldap;

use Ulmus\Ldap\Annotation\Classes\{ ObjectClass, ObjectType };
use Ulmus\{EntityCollection, Ulmus, SearchRequest, Query};
use Ulmus\Annotation\Property\{ Where, Having, Relation, Join, WithJoin, Relation\Ignore as RelationIgnore };
use Ulmus\Common\EntityResolver;

class Repository extends \Ulmus\Repository
{
    use Repository\ConditionTrait;

    const DEFAULT_ALIAS = "";

    public array $events = [];

    public function __construct(string $entity, string $alias = self::DEFAULT_ALIAS, ConnectionAdapter $adapter = null) {
        parent::__construct($entity, $alias, $adapter);

        $this->queryBuilder = new QueryBuilder();
    }

    protected function selectSqlQuery() : self
    {
        if ( null === $this->queryBuilder->getFragment(Query\Select::class) ) {
            $this->select(array_keys($this->entityResolver->fieldList(EntityResolver::KEY_COLUMN_NAME)));
        }

        if ( null !== $objectClass = $this->entityResolver->getAnnotationFromClassname( ObjectClass::class, false ) ) {
            $this->where('objectclass', $objectClass->type);
        }

        if ( null !== $objectClass = $this->entityResolver->getAnnotationFromClassname( ObjectType::class, false ) ) {
            $this->where('objecttype', $objectClass->type);
        }

        return $this;
    }

    protected function updateSqlQuery(array $dataset) : self
    {
        $condition = array_pop($this->queryBuilder->where->conditionList);

        if ($condition[0] === 'dn') {
            if ( null === $this->queryBuilder->getFragment(Query\Update::class) ) {
                $this->update($condition[1], "", "");
            }
        }
        else {
            array_push($this->queryBuilder->where->conditionList, $condition);
        }

        $this->set($dataset);

        return $this;
    }

    public function update(string $dn, string $alias, ? string $schema) : self
    {
        $this->queryBuilder->update($dn, "", "");

        return $this;
    }

    public function escapeValue(string $identifier) : string
    {
        return $this->adapter->adapter()->escapeIdentifier($identifier, Adapter\Ldap::IDENTIFIER_FILTER);
    }

    public function filterServerRequest(SearchRequest\SearchRequestInterface $searchRequest, bool $count = true) : self
    {
        if ($count) {
            $this->eventRegister(new class($searchRequest) implements \Ulmus\Event\Repository\CollectionFromQueryInterface {

                protected SearchRequest\SearchRequestInterface $searchRequest;

                public function __construct(SearchRequest\SearchRequestInterface $searchRequest) {
                    $this->searchRequest = $searchRequest;
                }

                public function execute(EntityCollection $collection) : EntityCollection
                {
                    $this->searchRequest->count = $collection->count();

                    return $collection;
                }
            });
        }

        return parent::filterServerRequest($searchRequest, false);
    }
}