<?php

namespace Ulmus\Ldap\Repository;

use Ulmus\Query;

use Ulmus\Repository;

use Ulmus\Ldap\Query\Filter;

trait ConditionTrait 
{
    public function where($field, $value, string $operator = Query\Where::OPERATOR_EQUAL, $condition = Query\Where::CONDITION_AND) : Repository
    {
        $this->queryBuilder->where($this->escapeField($field), $this->escapeValue($value), $operator, $condition);

        return $this;
    }

    public function wheres(array $fieldValues, string $operator = Query\Where::OPERATOR_EQUAL) : Repository
    {
        foreach($fieldValues as $field => $value) {
            $this->where($field, $value, $operator);
        }
        
        return $this;
    }
    
    public function and($field, $value, string $operator = Query\Where::OPERATOR_EQUAL) : Repository
    {
        return $this->where($field, $value, $operator);
    }

    public function or($field, $value, string $operator = Query\Where::OPERATOR_EQUAL) : Repository
    {
        $this->queryBuilder->where($this->escapeField($field), $this->escapeValue($value), $operator, Query\Where::CONDITION_OR);
        
        return $this;
    }

    public function notWhere($field, $value, string $operator = Query\Where::OPERATOR_NOT_EQUAL) : Repository
    {
        $this->queryBuilder->where($this->escapeField($field), $this->escapeValue($value), $operator, Query\Where::CONDITION_AND, true);
        
        return $this;
    }

    public function orNot($field, $value, string $operator = Query\Where::OPERATOR_NOT_EQUAL) : Repository
    {
        $this->queryBuilder->notWhere($this->escapeField($field), $this->escapeValue($value), $operator, Query\Where::CONDITION_OR, true);

        return $this;
    }

    public function having($field, $value, string $operator = Query\Where::OPERATOR_EQUAL, $condition = Query\Where::CONDITION_AND) : Repository
    {
        $this->where($field, $value, $operator, $condition);
        
        return $this;
    }

    public function orHaving($field, $value, string $operator = Query\Where::OPERATOR_EQUAL) : Repository
    {
        $this->orWhere($field, $value, $operator, Query\Where::CONDITION_OR);
        
        return $this;
    }

    public function notHaving($field, $value, string $operator = Query\Where::OPERATOR_NOT_EQUAL) : Repository
    {
        $this->queryBuilder->where($this->escapeField($field), $this->escapeValue($value), $operator, Query\Where::CONDITION_AND, true);
        
        return $this;
    }

    public function orNotHaving($field, $value) : Repository
    {
        $this->queryBuilder->where($this->escapeField($field), $this->escapeValue($value), Query\Where::OPERATOR_NOT_EQUAL, Query\Where::CONDITION_OR, true);
        
        return $this;
    }
    
    public function in($field, $value, string $operator = Query\Where::OPERATOR_EQUAL) : Repository
    {
//        $this->queryBuilder->where($field, $value, $operator);
        
        return $this;
    }

    public function orIn($field, $value, string $operator = Query\Where::OPERATOR_EQUAL) : Repository
    {
//        $this->queryBuilder->where($field, $value, $operator, Query\Where::CONDITION_OR);
        
        return $this;
    }

    public function notIn($field, $value) : Repository
    {
//        $this->queryBuilder->where($field, $value, Query\Where::OPERATOR_NOT_EQUAL);
        
        return $this;
    }

    public function orNotIn($field, $value) : Repository
    {
//        return $this->orNot($field, $value, Query\Where::OPERATOR_NOT_EQUAL, Query\Where::CONDITION_OR, true);
    }

    public function like($field, $value) : Repository
    {
        $this->where($field, $value, Filter::OPERATOR_LIKE);
        
        return $this;
    }

    public function orLike($field, $value) : Repository
    {
        $this->or($field, $value, Filter::OPERATOR_LIKE);
        
        return $this;
    }

    public function notLike($field, $value) : Repository
    {
        $this->notWhere($field, $value, Filter::OPERATOR_LIKE);
        
        return $this;
    }

    public function likes(array $fieldValues, string $condition = Query\Where::CONDITION_AND) : Repository
    {
        foreach($fieldValues as $field => $value) {
            $this->like($field, $value);
        }
        
        return $this;
    }
}